"""Here we import this modules implementation classes with generic names for the
Factory to use"""
from ._mockrepo import MockFideliusRepo as FideliusRepo
from ._mockadmin import MockFideliusAdmin as FideliusAdmin
